local vector = require 'lib.vector'

local Kinematic = Class.Game.Physical:extend()

function Kinematic:new(world, pos, size)
  Class.Game.Physical.new(self, world, pos, size)

  self.gravity = 2000

  self.standing = false
end

function Kinematic:filter(col)
  if col.other:is(Class.Game.Wall) and col.other ~= self.standing then
    if col.normal.x ~= 0 then
      self.velocity.x = 0
    end
    if col.normal.y ~= 0 then
      self.velocity.y = 0
      if col.normal.y < 0 then
        self.standing = col.other
      end
    end
    return true
  end

  if col.other:is(Class.Game.Box) and col.other ~= self.standing then
    if col.normal.y ~= 0 then
      if col.normal.y < 0 and self.velocity.y > 0 then
        self.velocity.y = 0
        self.standing = col.other
      end
      return true
    end
  end
end

function Kinematic:update(dt)
  if self.standing then
    --move with moving objects
    --self:move(self.standing:getFinalVelocity() * dt)

    --check if no longer standing
    local new = self:getPos() + vector(0, 1)
    _, _, _, len = self.world:check(self, new.x, new.y, function()
      return 'cross'
    end)
    if len == 0 then
      self.standing = false
    end
  else
    --apply gravity
    self.velocity.y = self.velocity.y + self.gravity * dt
  end

  Class.Game.Physical.update(self, dt)
end

return Kinematic
