local vector = require 'lib.vector'

local Physical = Class.Game.Common:extend()

function Physical:new(world, pos, size)
  self.world = world
  world:add(self, pos.x, pos.y, size.x, size.y)
  self.velocity = vector()
end

function Physical:filter(col)
  return false
end

function Physical:getPos()
  x, y = self.world:getRect(self)
  return vector(x, y)
end

function Physical:getSize()
  _, _, w, h = self.world:getRect(self)
  return vector(w, h)
end

function Physical:getCenter()
  return self:getPos() + self:getSize() / 2
end

function Physical:getFinalVelocity()
  local v = self.velocity
  if self.standing then
    v = v + self.standing:getFinalVelocity()
  end
  return v
end

--[[This move function wraps bump's move function, but uses a different
filter system. Physical:filter accepts collision information and returns
whether "slide" resolution should be used for that particular collision.]]
function Physical:move(dv)
  local goal = self:getPos() + dv

  --check for all collisions
  _, _, cols = self.world:check(self, goal.x, goal.y, function()
    return 'cross'
  end)

  --look for a collision that should be resolved by bump (defined by filter)
  local moved = false
  for _, col in pairs(cols) do
    if self:filter(col) then
      self.world:move(self, goal.x, goal.y, function(item, other)
        if other == col.other then
          return 'slide'
        else
          return false
        end
      end)
      moved = true
    end
  end

  --move if none of the colliding objects matched the filter
  if not moved then
    self.world:move(self, goal.x, goal.y, function() return false end)
  end
end

function Physical:update(dt)
  self:move(vector())
  self:move(self:getFinalVelocity() * dt)
end

function Physical:drawHitbox(color)
  color = color or {150, 150, 150}
  love.graphics.setColor(color)
  love.graphics.rectangle('fill', self.world:getRect(self))
end

function Physical:draw()
  self:drawHitbox()
end

return Physical
