local vector = require 'lib.vector'
local lume   = require 'lib.lume'

local Player = Class.Game.Kinematic:extend()

function Player:new(world, pos)
  Class.Game.Kinematic.new(self, world, pos, vector(50, 50))

  self.acceleration = 2000
  self.walkSpeed    = 400
  self.friction     = 10
  self.jumpPower    = 750
end

function Player:update(dt)
  --walking
  local vx = self.velocity.x
  vx = vx + self.acceleration * Control.Axis.Horizontal:getValue() * dt
  if Control.Axis.Horizontal:getValue() == 0 then
    vx = lume.lerp(vx, 0, 1 - (10 ^ -self.friction) ^ dt)
  end
  vx = lume.clamp(vx, -self.walkSpeed, self.walkSpeed)
  self.velocity.x = vx

  --jumping
  if self.standing and Control.Button.Primary:pressed() then
    self.velocity.y = -self.jumpPower
  end

  Class.Game.Kinematic.update(self, dt)
end

function Player:draw()
  self:drawHitbox {255, 255, 255}
  love.graphics.setColor(255, 255, 255)
  love.graphics.print(tostring(self.velocity))
  love.graphics.print(tostring(self.standing), 0, 20)
end

return Player
